__author__ = "Matthias Buechele"
__author_email__ = "matthias.buechele@fau.de"
__maintainer__ = __author__
__email__ = __author_email__
__copyright__ = "Copyright 2017, Matthias Buechele"
__credits__ = ["Matthias Buechele"]
__license__ = "GNU GPL v3"
__shortname__ = "nsb"
__description__ = 'Draws nightsky allskymaps',
__long_description__ = ("Draws nightsky allskymaps corresponding to KRISCIUNAS Model of the Brightness of Moonlight,\n"
                "together with star data obtained from the GAIA public data release catalog.\n"
                "The result is a 2D Pixel array with physical brightness values for each sky position.")
__status__ = "Prototype"  # "Prototype", "Development", or "Production"
__version__ = "0.2.0"
